@@ -35,7 +35,6 @@
 
 #include "debug.h"
 #include "lib/util/signal.h" /* Avoid /usr/include/signal.h */
-#include "substitute.h"
 #include "fault.h"
 
 static struct {
@@ -71,17 +70,15 @@ report a fault
 static void fault_report(int sig)
 {
 	static int counter;
+	char signal_string[128];
 
 	if (counter) _exit(1);
 
 	counter++;
 
-	DEBUGSEP(0);
-	DEBUG(0,("INTERNAL ERROR: Signal %d in pid %d (%s)",sig,(int)getpid(),SAMBA_VERSION_STRING));
-	DEBUG(0,("\nPlease read the Trouble-Shooting section of the Samba HOWTO\n"));
-	DEBUGSEP(0);
-
-	smb_panic("internal error");
+	snprintf(signal_string, sizeof(signal_string),
+		 "Signal %d: %s", sig, strsignal(sig));
+	smb_panic(signal_string);
 
 	/* smb_panic() never returns, so this is really redundant */
 	exit(1);
@@ -124,10 +121,6 @@ _PUBLIC_ const char *panic_action = NULL;
 static void smb_panic_default(const char *why) _NORETURN_;
 static void smb_panic_default(const char *why)
 {
-	DBG_ERR("PANIC (pid %llu): %s\n",
-		    (unsigned long long)getpid(), why);
-	log_stack_trace();
-
 #if defined(HAVE_PRCTL) && defined(PR_SET_PTRACER)
 	/*
 	 * Make sure all children can attach a debugger.
@@ -140,8 +133,22 @@ static void smb_panic_default(const char *why)
 		if (strlcpy(cmdstring, panic_action, sizeof(cmdstring)) < sizeof(cmdstring)) {
 			int result;
 			char pidstr[20];
+			char subst[200];
+			char *p = NULL;
 			snprintf(pidstr, sizeof(pidstr), "%d", (int) getpid());
-			all_string_sub(cmdstring, "%d", pidstr, sizeof(cmdstring));
+
+			p = strstr(cmdstring, "%d");
+			if (p != NULL) {
+				snprintf(subst,
+					 sizeof(subst),
+					 "%.*s%s%s",
+					 (int)(p-cmdstring),
+					 cmdstring,
+					 pidstr,
+					 p+2);
+				strlcpy(cmdstring, subst, sizeof(cmdstring));
+			}
+
 			DEBUG(0, ("smb_panic(): calling panic action [%s]\n", cmdstring));
 			result = system(cmdstring);
 
@@ -160,12 +167,32 @@ static void smb_panic_default(const char *why)
 	abort();
 }
 
+_PUBLIC_ void smb_panic_log(const char *why)
+{
+	DEBUGSEP(0);
+	DEBUG(0,("INTERNAL ERROR: %s in pid %lld (%s)\n",
+		 why,
+		 (unsigned long long)getpid(),
+		 SAMBA_VERSION_STRING));
+	DEBUG(0,("If you are running a recent Samba version, and "
+		 "if you think this problem is not yet fixed in the "
+		 "latest versions, please consider reporting this "
+		 "bug, see "
+		 "https://wiki.samba.org/index.php/Bug_Reporting\n"));
+	DEBUGSEP(0);
+	DEBUG(0,("PANIC (pid %llu): %s in " SAMBA_VERSION_STRING "\n",
+		 (unsigned long long)getpid(), why));
+
+	log_stack_trace();
+}
 
 /**
    Something really nasty happened - panic !
 **/
 _PUBLIC_ void smb_panic(const char *why)
 {
+	smb_panic_log(why);
+
 	if (fault_state.panic_handler) {
 		fault_state.panic_handler(why);
 		_exit(1);
@@ -261,10 +288,10 @@ libunwind_failed:
 		  (unsigned long)backtrace_size));
 
 	if (backtrace_strings) {
-		int i;
+		size_t i;
 
 		for (i = 0; i < backtrace_size; i++)
-			DEBUGADD(0, (" #%u %s\n", i, backtrace_strings[i]));
+			DEBUGADD(0, (" #%zu %s\n", i, backtrace_strings[i]));
 
 		/* Leak the backtrace_strings, rather than risk what free() might do */
 	}
